#build/testing/testing_cgegqr_gpu
#build/testing/testing_cgelqf_gpu
#build/testing/testing_cgels3_gpu
#build/testing/testing_cgels_gpu
#build/testing/testing_cgeqp3_gpu
#build/testing/testing_cgeqr2_gpu
#build/testing/testing_cgeqr2x_gpu
#build/testing/testing_cgeqrf_gpu
#build/testing/testing_cgeqrf_mgpu
#build/testing/testing_cgesv_gpu
#build/testing/testing_cgetf2_gpu
#build/testing/testing_cgetrf_gpu
#build/testing/testing_cgetrf_mgpu
#build/testing/testing_cgetri_gpu
#build/testing/testing_cheevd_gpu
#build/testing/testing_chegst_gpu
#build/testing/testing_chemm_mgpu
#build/testing/testing_chemv_mgpu
#build/testing/testing_cher2k_mgpu
#build/testing/testing_chesv_nopiv_gpu
#build/testing/testing_chetrd_gpu
#build/testing/testing_chetrd_mgpu
#build/testing/testing_clarfb_gpu
#build/testing/testing_cposv_gpu
#build/testing/testing_cpotf2_gpu
#build/testing/testing_cpotrf_gpu
#build/testing/testing_cpotrf_mgpu
#build/testing/testing_cpotri_gpu
#build/testing/testing_csysv_nopiv_gpu
#build/testing/testing_ctrtri_gpu
#build/testing/testing_cungqr_gpu
#build/testing/testing_cunmql_gpu
#build/testing/testing_cunmqr_gpu
#build/testing/testing_dgegqr_gpu
#build/testing/testing_dgelqf_gpu
#build/testing/testing_dgels3_gpu
#build/testing/testing_dgels_gpu
#build/testing/testing_dgeqp3_gpu
#build/testing/testing_dgeqr2_gpu
#build/testing/testing_dgeqr2x_gpu
#build/testing/testing_dgeqrf_gpu
#build/testing/testing_dgeqrf_mgpu
#build/testing/testing_dgesv_gpu
#build/testing/testing_dgetf2_gpu
#build/testing/testing_dgetrf_gpu
#build/testing/testing_dgetrf_mgpu
#build/testing/testing_dgetri_gpu
#build/testing/testing_dlarfb_gpu
#build/testing/testing_dorgqr_gpu
#build/testing/testing_dormql_gpu
#build/testing/testing_dormqr_gpu
#build/testing/testing_dposv_gpu
#build/testing/testing_dpotf2_gpu
#build/testing/testing_dpotrf_gpu
#build/testing/testing_dpotrf_mgpu
#build/testing/testing_dpotri_gpu
#build/testing/testing_dsgeqrsv_gpu
#build/testing/testing_dsgesv_gpu
#build/testing/testing_dsposv_gpu
#build/testing/testing_dsyevd_gpu
#build/testing/testing_dsygst_gpu
#build/testing/testing_dsymm_mgpu
#build/testing/testing_dsymv_mgpu
#build/testing/testing_dsyr2k_mgpu
#build/testing/testing_dsysv_nopiv_gpu
#build/testing/testing_dsytrd_gpu
#build/testing/testing_dsytrd_mgpu
#build/testing/testing_dtrtri_gpu
#build/testing/testing_sgegqr_gpu
#build/testing/testing_sgelqf_gpu
#build/testing/testing_sgels3_gpu
#build/testing/testing_sgels_gpu
#build/testing/testing_sgeqp3_gpu
#build/testing/testing_sgeqr2_gpu
#build/testing/testing_sgeqr2x_gpu
#build/testing/testing_sgeqrf_gpu
#build/testing/testing_sgeqrf_mgpu
#build/testing/testing_sgesv_gpu
#build/testing/testing_sgetf2_gpu
#build/testing/testing_sgetrf_gpu
#build/testing/testing_sgetrf_mgpu
#build/testing/testing_sgetri_gpu
#build/testing/testing_slarfb_gpu
#build/testing/testing_sorgqr_gpu
#build/testing/testing_sormql_gpu
#build/testing/testing_sormqr_gpu
#build/testing/testing_sposv_gpu
#build/testing/testing_spotf2_gpu
#build/testing/testing_spotrf_gpu
#build/testing/testing_spotrf_mgpu
#build/testing/testing_spotri_gpu
#build/testing/testing_ssyevd_gpu
#build/testing/testing_ssygst_gpu
#build/testing/testing_ssymm_mgpu
#build/testing/testing_ssymv_mgpu
#build/testing/testing_ssyr2k_mgpu
#build/testing/testing_ssysv_nopiv_gpu
#build/testing/testing_ssytrd_gpu
#build/testing/testing_ssytrd_mgpu
#build/testing/testing_strtri_gpu
#build/testing/testing_zcgeqrsv_gpu
#build/testing/testing_zcgesv_gpu
#build/testing/testing_zcposv_gpu
#build/testing/testing_zgegqr_gpu
#build/testing/testing_zgelqf_gpu
#build/testing/testing_zgels3_gpu
#build/testing/testing_zgels_gpu
#build/testing/testing_zgeqp3_gpu
#build/testing/testing_zgeqr2_gpu
#build/testing/testing_zgeqr2x_gpu
#build/testing/testing_zgeqrf_gpu
#build/testing/testing_zgeqrf_mgpu
#build/testing/testing_zgesv_gpu
#build/testing/testing_zgetf2_gpu
#build/testing/testing_zgetrf_gpu
#build/testing/testing_zgetrf_mgpu
#build/testing/testing_zgetri_gpu
#build/testing/testing_zheevd_gpu
#build/testing/testing_zhegst_gpu
#build/testing/testing_zhemm_mgpu
#build/testing/testing_zhemv_mgpu
#build/testing/testing_zher2k_mgpu
#build/testing/testing_zhesv_nopiv_gpu
#build/testing/testing_zhetrd_gpu
#build/testing/testing_zhetrd_mgpu
#build/testing/testing_zlarfb_gpu
#build/testing/testing_zposv_gpu
#build/testing/testing_zpotf2_gpu
#build/testing/testing_zpotrf_gpu
#build/testing/testing_zpotrf_mgpu
#build/testing/testing_zpotri_gpu
#build/testing/testing_zsysv_nopiv_gpu
#build/testing/testing_ztrtri_gpu
#build/testing/testing_zungqr_gpu
#build/testing/testing_zunmql_gpu
#build/testing/testing_zunmqr_gpu
